"""
bass.setting
-----
Global settings, shared by all modules.
"""

extension    = None
follow_links = None
host         = None
ignore       = None
input        = None
layout       = None
output       = None
port         = None
project      = None
root_url     = None
template     = None
version      = '1.0.0'
